/*
 * Copyright (C) 2007 Ted Bullock <tbullock@canada.com>
 * 
 * This file is part of httperf, a web server performance measurment tool.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * In addition, as a special exception, the copyright holders give permission
 * to link the code of this work with the OpenSSL project's "OpenSSL" library
 * (or with modified versions of it that use the same license as the "OpenSSL" 
 * library), and distribute linked combinations including the two.  You must
 * obey the GNU General Public License in all respects for all of the code
 * used other than "OpenSSL".  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>		/* For strrchr() */
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

#include <sys/time.h>
#include <event.h>
#include <evdns.h>

#include <generic_types.h>
#include <list.h>

/*
 * Events allocated onto the heap
 */
static struct List *active_events = NULL;
static struct List *inactive_events = NULL;

static const char *prog_name = NULL;
static unsigned long num_conn = 0, num_closed = 0;
static struct timeval start_time;

static char    *server = NULL;
static int      desired = 0;
static int      port = 0;

static void
cleanup()
{
	if (active_events != NULL) {
		while (!is_list_empty(active_events)) {
			Any_Type        a = list_pop(active_events);
			free(a.vp);
		}
		list_free(active_events);
	}
}

void
sigint_exit(int fd, short event, void *arg)
{
	struct event   *signal_int = arg;
	struct timeval  stop_time;
	double          delta_t = 0;

	gettimeofday(&stop_time, NULL);

	delta_t = ((stop_time.tv_sec - start_time.tv_sec)
		   + 1e-6 * (stop_time.tv_usec - start_time.tv_usec));

	printf("%s: total # conn. created = %lu, close() rate = %g conn/sec\n",
	       prog_name, num_conn, num_closed / delta_t);

	printf("%s: caught SIGINT... Exiting.\n", __func__);

	evdns_shutdown(0);
	event_del(signal_int);

	cleanup();
}

void
dns_lookup_callback(int result, char type, int count, int ttl, void *addresses,
		    void *arg)
{
	uint8_t         i;

	if (result != DNS_ERR_NONE) {
		printf("DNS Lookup: result(%d)\n",
		       evdns_err_to_string(result));
		exit(EXIT_FAILURE);
	}

	printf("Resolved %s\n", server);

	for (i = 0; i < count; ++i) {
		uint32_t       *address_list = (uint32_t *) addresses;
		uint8_t         oct[4];
		oct[0] = (uint8_t) (address_list[i] & 0x000000ff);
		oct[1] = (uint8_t) ((address_list[i] & 0x0000ff00) >> 8);
		oct[2] = (uint8_t) ((address_list[i] & 0x00ff0000) >> 16);
		oct[3] = (uint8_t) ((address_list[i] & 0xff000000) >> 24);
		printf("\t(%u.%u.%u.%u)\n", oct[0], oct[1], oct[2], oct[3]);
	}


}

void
main(int argc, char *argv[])
{
	struct event    signal_int;

	active_events = list_create();
	if (active_events == NULL)
		goto init_failure;

	inactive_events = list_create();
	if (inactive_events == NULL)
		goto init_failure;

	event_init();
	evdns_init();

	/*
	 * Initalize one event 
	 */
	event_set(&signal_int, SIGINT, EV_SIGNAL, sigint_exit, &signal_int);

	event_add(&signal_int, NULL);

	prog_name = strrchr(argv[0], '/');
	if (prog_name)
		++prog_name;
	else
		prog_name = argv[0];

	if (argc != 4) {
		fprintf(stderr, "Usage: `%s server port numidle'\n",
			prog_name);
		goto init_failure;
	}

	server = argv[1];
	port = atoi(argv[2]);
	desired = atoi(argv[3]);

	printf("%s: Using libevent-%s for %s event notification system.\n"
	       "Control-c to exit\n\n", prog_name, event_get_version(),
	       event_get_method());

	gettimeofday(&start_time, NULL);

	evdns_resolve_ipv4(server, 0, dns_lookup_callback, NULL);

	event_dispatch();

	/*
	 * Event loop will only exit upon receiving SIGINT.  Make sure we pass
	 * this on to the parent process 
	 */
	if (signal(SIGINT, SIG_DFL) < 0)
		perror("signal");
	else
		kill(getpid(), SIGINT);

      init_failure:
	cleanup();
	exit(EXIT_FAILURE);
}
